<?php

/**
 * @file
 * Contains class CurrencyLayerCurrencyExchanger.
 */

/**
 * Retrieves currency exchange rates from Currencylayer.com.
 */
class CurrencyLayerCurrencyExchanger implements CurrencyExchangerInterface {

  static function getEndpoint() {
    return 'http://apilayer.net/api/';
  }

  /**
   * Implements CurrencyExchangerInterface::load().
   */
  static function load($currency_code_from, $currency_code_to) {
    // We deliberately run this through loadMultiple() rather than request(),
    // because loadMultiple() caches the rates.
    $rates = self::loadMultiple(array(
          $currency_code_from => array($currency_code_to),
    ));

    return $rates[$currency_code_from][$currency_code_to];
  }

  /**
   * Implements CurrencyExchangerInterface::loadMultiple().
   */
  static function loadMultiple(array $currency_codes) {
    $set_cache = FALSE;
    $cached_rates = self::getCache('currency_layer');
    $rates = array();
    $request = array();
    foreach ($currency_codes as $currency_code_from => $currency_codes_to) {
      foreach ($currency_codes_to as $currency_code_to) {
        if (empty($cached_rates[$currency_code_from]) || empty($cached_rates[$currency_code_from][$currency_code_to])) {
          $set_cache = TRUE;
          if (in_array($currency_code_from, self::currencyCodes()) && in_array($currency_code_to, self::currencyCodes())) {
            $request[$currency_code_from][] = $currency_code_to;
          }
          else {
            $cached_rates[$currency_code_from][$currency_code_to] = FALSE;
          }
        }
        else {
          $rates[$currency_code_from][$currency_code_to] = $cached_rates[$currency_code_from][$currency_code_to];
        }
      }
    }
    if ($request) {
      $response = self::getExchange($request);
      self::addToCache($cached_rates, $response);
    }
    if ($set_cache) {
      self::setCache('currency_layer', $cached_rates);
    }

    return $rates;
  }

  /**
   * Implements CurrencyExchangerInterface::operationsLinks().
   */
  static function operationsLinks() {
    return array(array(
        'title' => t('configure'),
        'href' => 'admin/config/regional/currency-exchange/currency-layer',
    ));
  }

  /**
   * Build exchange query for request.
   *
   * @param array $currency_codes
   *   Array of currency codes for rate result.
   */
  static function getExchange(array $currency_codes) {
    foreach ($currency_codes as $from => $tos) {
      $query = array(
        'currencies' => implode(',', $tos),
        'source' => $from,
        'format' => 1
      );
      $q = '&' . drupal_http_build_query($query);
      $response = self::request('live', $q);
      return self::perseResponse($response);
    }
  }

  /**
   * Format response object in currency from->to format.
   *
   * @param object $response
   *   API rate response object.
   * @return array
   *   Formated array containing the rates.
   */
  protected static function perseResponse($response) {
    $r = array();
    if (isset($response->quotes)) {
      foreach ((array) $response->quotes as $key => $value) {
        $r[$response->source][substr($key, 3)] = $value;
      }
    }
    return $r;
  }

  /**
   * Update the cache array with new values.
   *
   * @param array $cache
   *   Currency exchange cache.
   * @param array $data
   *   New data to update.
   */
  protected static function addToCache(&$cache, $data) {
    foreach ($data as $from => $to_array) {
      foreach ($to_array as $to => $value) {
        $cache[$from][$to] = $value;
      }
    }
  }

  /**
   * Returns the codes of all currencies that Currency layer offers rates for.
   *
   * @return array
   *   ISO 4217 currency codes.
   */
  static function currencyCodes() {
    static $currency_codes;
    if (is_array($currency_codes)) {
      return $currency_codes;
    }

    $set_cache = FALSE;
    $cached_codes = self::getCache('currency_layer_codes');
    if (!$cached_codes) {
      $response = self::request('list');
      if ($response) {
        $set_cache = TRUE;
        $cached_codes = $response->currencies;
      }
    }
    if ($set_cache) {
      self::setCache('currency_layer_codes', (array) $cached_codes);
    }
    $currency_codes = array_keys((array) $cached_codes);
    return $currency_codes;
  }

  /**
   * Requests call to Currency layer API.
   *
   * @param string $type
   * @param string $query
   *
   * @return json|NULL
   *   A json object if success, FALSE not.
   */
  protected static function request($type, $query = NULL) {
    $url = self::getEndpoint();
    $response = drupal_http_request($url . '/' . $type . '?access_key=' . variable_get('currency_layer_api_key', '') . $query);
    if ($response->code == 200) {
      $data = json_decode($response->data);
      if (!$data->success) {
        self::log(json_encode($data->error));
        return FALSE;
      }
      return $data;
    }
    self::log();
  }

  /**
   * Log to watchdog the errors.
   *
   * @param object|string $log
   */
  protected static function log($log = NULL) {
    if (!$log) {
      $log = t('Can`t reach the API!');
    }
    watchdog('CurrencyLayerCurrencyExchanger', $log, WATCHDOG_ERROR);
  }

  /**
   * Gets data from the cache.
   *
   * @return array
   *   The requested cache data.
   */
  static function getCache($name) {
    $cache = cache_get($name);
    if (is_object($cache) && $cache->created + self::cacheLifetime() > REQUEST_TIME) {
      return $cache->data;
    }
    return array();
  }

  /**
   * Saves data to the cache.
   *
   * @param array $rates
   *   The saved cache data.
   */
  static function setCache($name, array $data) {
    return cache_set($name, $data, 'cache', time() + self::cacheLifetime());
  }

  /**
   * Returns the cache lifetime.
   *
   * @return int
   */
  static function cacheLifetime() {
    return (int) variable_get('currency_layer_cache_lifetime', CURRENCY_LAYER_CACHE_LIFETIME);
  }

}
